# Reference

<!-- DO NOT EDIT: This document was generated by Puppet Strings -->

## Table of Contents

### Classes

* [`openssl`](#openssl): Manage X.509 certificates, keys and Diffie-Hellman parameter files

### Defined types

* [`openssl::cacert`](#openssl--cacert): Manage an X.509 CA certificate file in PEM format
* [`openssl::cert`](#openssl--cert): Manage an X.509 certificate file in PEM format
* [`openssl::config`](#openssl--config): *DEPRECATED* Create OpenSSL config for a CSR
* [`openssl::csr`](#openssl--csr): *DEPRECATED* Create OpenSSL certificate signing request (CSR)
* [`openssl::dhparam`](#openssl--dhparam): *DEPRECATED* Manage Diffie-Hellman parameter files
* [`openssl::key`](#openssl--key): Manage an X.509 key file in PEM format

### Resource types

* [`openssl_cert`](#openssl_cert): Create an OpenSSL certificate from a Certificate Signing Request
* [`openssl_certutil`](#openssl_certutil): Manage trusted certificates in the system-wide NSS database
* [`openssl_dhparam`](#openssl_dhparam): Generate a file with Diffie-Hellman parameters
* [`openssl_genparam`](#openssl_genparam): Generate Diffie-Hellman or Elliptic Curve parameter file
* [`openssl_genpkey`](#openssl_genpkey): Generate OpenSSL private key files
* [`openssl_hash`](#openssl_hash): Manage a symbolic link using the certificate hash
* [`openssl_key`](#openssl_key): Create an OpenSSL private key
* [`openssl_request`](#openssl_request): Create and maintain an OpenSSL Certificate Signing Request
* [`openssl_selfsign`](#openssl_selfsign): Create an OpenSSL self-signed certificate
* [`openssl_signcsr`](#openssl_signcsr): Sign OpenSSL certificate signing request using a CA

### Data types

* [`Openssl::Extendedkeyusage`](#Openssl--Extendedkeyusage): Valid parameter values for the OpenSSL extendend key usage
* [`Openssl::Keyusage`](#Openssl--Keyusage): Valid parameter values for the OpenSSL keyusage

## Classes

### <a name="openssl"></a>`openssl`

Manage X.509 certificates, keys and Diffie-Hellman parameter files

#### Examples

##### Declaring the class

```puppet

class { 'openssl':
  cert_source_directory => '/etc/puppetlabs/code/private/certs',
}
```

##### Declaring the class and deploy a CA certificate

```puppet

class { 'openssl':
  cert_source_directory => '/etc/puppetlabs/code/private/certs',
  root_ca_certs         => [ 'ACME-Root-CA' ],
}
```

#### Parameters

The following parameters are available in the `openssl` class:

* [`cert_source_directory`](#-openssl--cert_source_directory)
* [`default_key_dir`](#-openssl--default_key_dir)
* [`default_cert_dir`](#-openssl--default_cert_dir)
* [`package_name`](#-openssl--package_name)
* [`package_ensure`](#-openssl--package_ensure)
* [`root_group`](#-openssl--root_group)
* [`ca_certs`](#-openssl--ca_certs)

##### <a name="-openssl--cert_source_directory"></a>`cert_source_directory`

Data type: `Stdlib::Absolutepath`

The directory on the Puppetmaster where all certificate and key files are
kept. Every certificate or key file will be read from this directory and
then deployed on the client. This directory is accessed using the `file`
function and therefore does not need to be part of the Puppet directory
structure. But obviously the directory and the files must be readable by
the Puppet user. This parameter is mandatory and has no default.

##### <a name="-openssl--default_key_dir"></a>`default_key_dir`

Data type: `Stdlib::Absolutepath`

The default directory where a key file is deployed. This is operating
system specific. On Debian this is `/etc/ssl/private` and on RedHat this
is `/etc/pki/tls/private`.

##### <a name="-openssl--default_cert_dir"></a>`default_cert_dir`

Data type: `Stdlib::Absolutepath`

The default directory where a certificate file is deployed. This is
operating system specific. On Debian this is `/etc/ssl/certs` and on
RedHat this is `/etc/pki/tls/certs`.

##### <a name="-openssl--package_name"></a>`package_name`

Data type: `String`

The name of the OpenSSL package to install.

##### <a name="-openssl--package_ensure"></a>`package_ensure`

Data type: `String`

The desired package state.

##### <a name="-openssl--root_group"></a>`root_group`

Data type: `String`

The group used for deployed files. This is operating system specific. On
Linux this is normally `root`. On FreeBSD this is `wheel`.

##### <a name="-openssl--ca_certs"></a>`ca_certs`

Data type: `Array[String]`

An array of CA certificates that are installed by default. Internally
this uses the `openssl::cert` defined type.

## Defined types

### <a name="openssl--cacert"></a>`openssl::cacert`

Manage an X.509 CA certificate file in PEM format

#### Examples

##### Install the 'my-root-ca' trusted cert in the default location

```puppet

openssl::cacert { 'my-root-ca': }
```

#### Parameters

The following parameters are available in the `openssl::cacert` defined type:

* [`ensure`](#-openssl--cacert--ensure)
* [`cert`](#-openssl--cacert--cert)
* [`source`](#-openssl--cacert--source)
* [`extension`](#-openssl--cacert--extension)
* [`source_extension`](#-openssl--cacert--source_extension)
* [`mode`](#-openssl--cacert--mode)
* [`owner`](#-openssl--cacert--owner)
* [`group`](#-openssl--cacert--group)
* [`cert_dir`](#-openssl--cacert--cert_dir)

##### <a name="-openssl--cacert--ensure"></a>`ensure`

Data type: `Enum['present','absent']`

The state of the resource.

Default value: `'present'`

##### <a name="-openssl--cacert--cert"></a>`cert`

Data type: `String`

The basename of the file where the certificate will be stored on the
client. The full filename will be created using the three components
`cert_dir`, `cert` and `extension`.

Default value: `$name`

##### <a name="-openssl--cacert--source"></a>`source`

Data type: `String`

The basename of the file where the certificate is stored on the server.
The full filename will be created using the three parameters
`cert_source_directory` (see the base class `openssl`), `source` and
`source_extension`.

Default value: `$name`

##### <a name="-openssl--cacert--extension"></a>`extension`

Data type: `String`

The file extension used for files created on the client. This parameter
is ignored on Debian and RedHat based distributions as the operating
system specific tools require certificates to be installed using the
`.crt` extension.

Default value: `'crt'`

##### <a name="-openssl--cacert--source_extension"></a>`source_extension`

Data type: `String`

The file extension used for files read on the server.

Default value: `'crt'`

##### <a name="-openssl--cacert--mode"></a>`mode`

Data type: `Stdlib::Filemode`

The file mode used for the resource. Note that certificate
verification may fail if the file permissions are too restrictive.

Default value: `'0444'`

##### <a name="-openssl--cacert--owner"></a>`owner`

Data type: `String`

The file owner used for the resource.

Default value: `'root'`

##### <a name="-openssl--cacert--group"></a>`group`

Data type: `Optional[String]`

The file group used for the resource.

Default value: `undef`

##### <a name="-openssl--cacert--cert_dir"></a>`cert_dir`

Data type: `Optional[Stdlib::Absolutepath]`

The destination directory on the client where the certificate will be
stored. This parameter is ignored on Debian and RedHat based
distributions. Debian requires CA certificates to be stored in
`/usr/local/share/ca-certificates` and RedHat requires CA certificates
to be stored in `/etc/pki/ca-trust/source/anchors`.

Default value: `undef`

### <a name="openssl--cert"></a>`openssl::cert`

Manage an X.509 certificate file in PEM format

#### Examples

##### Install the 'imap' cert in the default location

```puppet

openssl::cert { 'imap': }
```

##### Install the 'postgresql' cert using application specific defaults

```puppet

openssl::cert { 'postgresql':
  cert     => $facts['networking']['hostname'],
  owner    => 'root',
  group    => 'postgres',
  mode     => '0444',
  cert_dir => '/etc/postgresql',
  source   => $facts['networking']['hostname'],
}
```

#### Parameters

The following parameters are available in the `openssl::cert` defined type:

* [`ensure`](#-openssl--cert--ensure)
* [`cert`](#-openssl--cert--cert)
* [`source`](#-openssl--cert--source)
* [`cert_chain`](#-openssl--cert--cert_chain)
* [`extension`](#-openssl--cert--extension)
* [`source_extension`](#-openssl--cert--source_extension)
* [`mode`](#-openssl--cert--mode)
* [`owner`](#-openssl--cert--owner)
* [`group`](#-openssl--cert--group)
* [`cert_dir`](#-openssl--cert--cert_dir)

##### <a name="-openssl--cert--ensure"></a>`ensure`

Data type: `Enum['present','absent']`

The state of the resource.

Default value: `'present'`

##### <a name="-openssl--cert--cert"></a>`cert`

Data type: `String`

The basename of the file where the certificate will be stored on the
client. The full filename will be created using the three components
`cert_dir`, `cert` and `extension`.

Default value: `$name`

##### <a name="-openssl--cert--source"></a>`source`

Data type: `String`

The basename of the file where the certificate is stored on the server.
The full filename will be created using the three parameters
`cert_source_directory` (see the base class `openssl`), `source` and
`source_extension`.

Default value: `$name`

##### <a name="-openssl--cert--cert_chain"></a>`cert_chain`

Data type: `Array[String]`

An array of certificate names that are should be added to the certificate
file. This allows the generation of certificate chains to provide a full
verification path for the certificate if intermediate CAs are used. The
chain is included in the generated certificate file. The certificates
must be available in `cert_source_directory` on the server just like the
ordinary certificate.

Default value: `[]`

##### <a name="-openssl--cert--extension"></a>`extension`

Data type: `String`

The file extension used for files created on the client.

Default value: `'crt'`

##### <a name="-openssl--cert--source_extension"></a>`source_extension`

Data type: `String`

The file extension used for files read on the server.

Default value: `'crt'`

##### <a name="-openssl--cert--mode"></a>`mode`

Data type: `Stdlib::Filemode`

The file mode used for the resource.

Default value: `'0444'`

##### <a name="-openssl--cert--owner"></a>`owner`

Data type: `String`

The file owner used for the resource.

Default value: `'root'`

##### <a name="-openssl--cert--group"></a>`group`

Data type: `Optional[String]`

The file group used for the resource.

Default value: `undef`

##### <a name="-openssl--cert--cert_dir"></a>`cert_dir`

Data type: `Optional[Stdlib::Absolutepath]`

The destination directory on the client where the certificate will be
stored.

Default value: `undef`

### <a name="openssl--config"></a>`openssl::config`

*DEPRECATED* This defined type will be removed in the next major
release. The custom type [`openssl_request`](#openssl_request) can be used
to create a CSR without the need to have a config file.

#### Examples

##### Creating a config file for a CSR

```puppet

openssl::config { '/etc/ssl/www.example.com.cnf':
  common_name        => 'www.example.com',
  extended_key_usage => [ 'serverAuth', 'clientAuth' ],
}
```

#### Parameters

The following parameters are available in the `openssl::config` defined type:

* [`common_name`](#-openssl--config--common_name)
* [`config`](#-openssl--config--config)
* [`subject_alternate_names_dns`](#-openssl--config--subject_alternate_names_dns)
* [`subject_alternate_names_ip`](#-openssl--config--subject_alternate_names_ip)
* [`key_usage`](#-openssl--config--key_usage)
* [`extended_key_usage`](#-openssl--config--extended_key_usage)
* [`basic_constraints_ca`](#-openssl--config--basic_constraints_ca)
* [`owner`](#-openssl--config--owner)
* [`group`](#-openssl--config--group)
* [`country_name`](#-openssl--config--country_name)
* [`state_or_province_name`](#-openssl--config--state_or_province_name)
* [`locality_name`](#-openssl--config--locality_name)
* [`postal_code`](#-openssl--config--postal_code)
* [`street_address`](#-openssl--config--street_address)
* [`organization_name`](#-openssl--config--organization_name)
* [`organization_unit_name`](#-openssl--config--organization_unit_name)

##### <a name="-openssl--config--common_name"></a>`common_name`

Data type: `String`

The value of the X.509 `CN` attribute. This attribute is mandatory.

##### <a name="-openssl--config--config"></a>`config`

Data type: `Stdlib::Absolutepath`

The full path name of the OpenSSL configuration file that will be
created. It contains a minimal set of configuration options that are
needed to process a CSR.

Default value: `$name`

##### <a name="-openssl--config--subject_alternate_names_dns"></a>`subject_alternate_names_dns`

Data type: `Array[Stdlib::Fqdn]`

An array of DNS names that will be added as subject alternate names using
the `DNS` prefix. The certificate can be used for all names given in this
list. Normally the common name should be in this list or the certificate
may be rejected by modern web browsers.

Default value: `[]`

##### <a name="-openssl--config--subject_alternate_names_ip"></a>`subject_alternate_names_ip`

Data type: `Array[Stdlib::IP::Address]`

An array of IP addresses that will be added as subject alternate names
using the `IP` prefix. The certificate can be used for all IP addresses
given in this list.

Default value: `[]`

##### <a name="-openssl--config--key_usage"></a>`key_usage`

Data type: `Array[Openssl::Keyusage]`

The intended purposes of the certificate.

Default value: `['keyEncipherment', 'dataEncipherment']`

##### <a name="-openssl--config--extended_key_usage"></a>`extended_key_usage`

Data type: `Array[Openssl::Extendedkeyusage]`

The extended key usage of the certificate.

Default value: `['serverAuth']`

##### <a name="-openssl--config--basic_constraints_ca"></a>`basic_constraints_ca`

Data type: `Boolean`

Whether the subject of the certificate is a CA.

Default value: `false`

##### <a name="-openssl--config--owner"></a>`owner`

Data type: `String`

The file owner used for the resource.

Default value: `'root'`

##### <a name="-openssl--config--group"></a>`group`

Data type: `Optional[String]`

The file group used for the resource.

Default value: `undef`

##### <a name="-openssl--config--country_name"></a>`country_name`

Data type: `Optional[String]`

The value of the X.509 `C` attribute.

Default value: `undef`

##### <a name="-openssl--config--state_or_province_name"></a>`state_or_province_name`

Data type: `Optional[String]`

The value of the X.509 `ST` attribute.

Default value: `undef`

##### <a name="-openssl--config--locality_name"></a>`locality_name`

Data type: `Optional[String]`

The value of the X.509 `L` attribute.

Default value: `undef`

##### <a name="-openssl--config--postal_code"></a>`postal_code`

Data type: `Optional[String]`

The value of the X.509 `PC` attribute.

Default value: `undef`

##### <a name="-openssl--config--street_address"></a>`street_address`

Data type: `Optional[String]`

The value of the X.509 `STREET` attribute.

Default value: `undef`

##### <a name="-openssl--config--organization_name"></a>`organization_name`

Data type: `Optional[String]`

The value of the X.509 `O` attribute.

Default value: `undef`

##### <a name="-openssl--config--organization_unit_name"></a>`organization_unit_name`

Data type: `Optional[String]`

The value of the X.509 `OU` attribute.

Default value: `undef`

### <a name="openssl--csr"></a>`openssl::csr`

*DEPRECATED* This defined type will be removed in the next major
release. Use the custom type [`openssl_request`](#openssl_request) to
create certificate requests instead.

#### Examples

##### Creating a CSR with subject alternate names

```puppet

openssl::csr { '/etc/ssl/www.example.com.csr':
  common_name                 => 'www.example.com',
  subject_alternate_names_dns => [ 'www.example.com', 'example.com', ],
  config                      => '/etc/ssl/www.example.com.cnf',
  key_file                    => '/etc/ssl/www.example.com.key',
}
```

#### Parameters

The following parameters are available in the `openssl::csr` defined type:

* [`common_name`](#-openssl--csr--common_name)
* [`csr_file`](#-openssl--csr--csr_file)
* [`config`](#-openssl--csr--config)
* [`key_file`](#-openssl--csr--key_file)
* [`subject_alternate_names_dns`](#-openssl--csr--subject_alternate_names_dns)
* [`subject_alternate_names_ip`](#-openssl--csr--subject_alternate_names_ip)
* [`key_usage`](#-openssl--csr--key_usage)
* [`extended_key_usage`](#-openssl--csr--extended_key_usage)
* [`basic_constraints_ca`](#-openssl--csr--basic_constraints_ca)
* [`mode`](#-openssl--csr--mode)
* [`owner`](#-openssl--csr--owner)
* [`group`](#-openssl--csr--group)
* [`country_name`](#-openssl--csr--country_name)
* [`state_or_province_name`](#-openssl--csr--state_or_province_name)
* [`locality_name`](#-openssl--csr--locality_name)
* [`postal_code`](#-openssl--csr--postal_code)
* [`street_address`](#-openssl--csr--street_address)
* [`organization_name`](#-openssl--csr--organization_name)
* [`organization_unit_name`](#-openssl--csr--organization_unit_name)

##### <a name="-openssl--csr--common_name"></a>`common_name`

Data type: `String`

The value of the X.509 `CN` attribute. This attribute is mandatory.

##### <a name="-openssl--csr--csr_file"></a>`csr_file`

Data type: `Stdlib::Absolutepath`

The full path name of the signing request file that will be created. It
contains the attributes that will be included in the certificate and also
the public part of the key. Default is the name of the resource.

Default value: `$name`

##### <a name="-openssl--csr--config"></a>`config`

Data type: `Stdlib::Absolutepath`

The full path name of the OpenSSL configuration file that will be
created. It contains a minimal set of configuration options that are
needed to process the CSR. It can also be used when the CSR is used to
create a self-signed certificate. Updates to the config file will not
trigger the generation of a new certificate.

##### <a name="-openssl--csr--key_file"></a>`key_file`

Data type: `Stdlib::Absolutepath`

The full path of the private key file. This file including the key must
already be present to generate the CSR.

##### <a name="-openssl--csr--subject_alternate_names_dns"></a>`subject_alternate_names_dns`

Data type: `Array[Stdlib::Fqdn]`

An array of DNS names that will be added as subject alternate names using
the `DNS` prefix. The certificate can be used for all names given in this
list. Normally the common name should be in this list or the certificate
may be rejected by modern web browsers.

Default value: `[]`

##### <a name="-openssl--csr--subject_alternate_names_ip"></a>`subject_alternate_names_ip`

Data type: `Array[Stdlib::IP::Address]`

An array of IP addresses that will be added as subject alternate names
using the `IP` prefix. The certificate can be used for all IP addresses
given in this list.

Default value: `[]`

##### <a name="-openssl--csr--key_usage"></a>`key_usage`

Data type: `Array[Openssl::Keyusage]`

The intended purposes of the certificate.

Default value: `['keyEncipherment', 'dataEncipherment']`

##### <a name="-openssl--csr--extended_key_usage"></a>`extended_key_usage`

Data type: `Array[Openssl::Extendedkeyusage]`

The extended key usage of the certificate.

Default value: `['serverAuth']`

##### <a name="-openssl--csr--basic_constraints_ca"></a>`basic_constraints_ca`

Data type: `Boolean`

Whether the subject of the certificate is a CA.

Default value: `false`

##### <a name="-openssl--csr--mode"></a>`mode`

Data type: `Stdlib::Filemode`

The file mode used for the resource.

Default value: `'0444'`

##### <a name="-openssl--csr--owner"></a>`owner`

Data type: `String`

The file owner used for the resource.

Default value: `'root'`

##### <a name="-openssl--csr--group"></a>`group`

Data type: `Optional[String]`

The file group used for the resource.

Default value: `undef`

##### <a name="-openssl--csr--country_name"></a>`country_name`

Data type: `Optional[String]`

The value of the X.509 `C` attribute.

Default value: `undef`

##### <a name="-openssl--csr--state_or_province_name"></a>`state_or_province_name`

Data type: `Optional[String]`

The value of the X.509 `ST` attribute.

Default value: `undef`

##### <a name="-openssl--csr--locality_name"></a>`locality_name`

Data type: `Optional[String]`

The value of the X.509 `L` attribute.

Default value: `undef`

##### <a name="-openssl--csr--postal_code"></a>`postal_code`

Data type: `Optional[String]`

The value of the X.509 `PC` attribute.

Default value: `undef`

##### <a name="-openssl--csr--street_address"></a>`street_address`

Data type: `Optional[String]`

The value of the X.509 `STREET` attribute.

Default value: `undef`

##### <a name="-openssl--csr--organization_name"></a>`organization_name`

Data type: `Optional[String]`

The value of the X.509 `O` attribute.

Default value: `undef`

##### <a name="-openssl--csr--organization_unit_name"></a>`organization_unit_name`

Data type: `Optional[String]`

The value of the X.509 `OU` attribute.

Default value: `undef`

### <a name="openssl--dhparam"></a>`openssl::dhparam`

*DEPRECATED* This defined type will be removed in the next major
release. Use the custom type [`openssl_dhparam`](#openssl_dhparam) instead.

#### Examples

##### Create a parameter file using default parameters

```puppet

openssl::dhparam { '/etc/ssl/dhparam.pem': }
```

##### Create a parameter file using 4096 bits

```puppet

openssl::dhparam { '/etc/ssl/dhparam.pem':
  bits => '4096',
}
```

##### Create a parameter file using non-default permissions

```puppet

openssl::dhparam { '/etc/ssl/dhparam.pem':
  owner => 'www-data',
  group => 'www-data',
  mode  => '0640',
}
```

#### Parameters

The following parameters are available in the `openssl::dhparam` defined type:

* [`ensure`](#-openssl--dhparam--ensure)
* [`file`](#-openssl--dhparam--file)
* [`bits`](#-openssl--dhparam--bits)
* [`generator`](#-openssl--dhparam--generator)
* [`mode`](#-openssl--dhparam--mode)
* [`owner`](#-openssl--dhparam--owner)
* [`group`](#-openssl--dhparam--group)

##### <a name="-openssl--dhparam--ensure"></a>`ensure`

Data type: `Enum['present','absent']`

The state of the resource.

Default value: `'present'`

##### <a name="-openssl--dhparam--file"></a>`file`

Data type: `Stdlib::Absolutepath`

The file name where the DH parameters are stored on the node. Must be an
absolute path.

Default value: `$name`

##### <a name="-openssl--dhparam--bits"></a>`bits`

Data type: `Enum['2048','3072','4096','5120','6144','7168','8192']`

The number of bits to generate.

Default value: `'2048'`

##### <a name="-openssl--dhparam--generator"></a>`generator`

Data type: `Enum['2','5']`

The generator to use. Check the OpenSSL documentation for details about
this parameter.

Default value: `'2'`

##### <a name="-openssl--dhparam--mode"></a>`mode`

Data type: `Stdlib::Filemode`

The file mode used for the resource.

Default value: `'0644'`

##### <a name="-openssl--dhparam--owner"></a>`owner`

Data type: `String`

The file owner used for the resource.

Default value: `'root'`

##### <a name="-openssl--dhparam--group"></a>`group`

Data type: `Optional[String]`

The file group used for the resource.

Default value: `undef`

### <a name="openssl--key"></a>`openssl::key`

Manage an X.509 key file in PEM format

#### Examples

##### Install the 'imap' key in the default location

```puppet

openssl::key { 'imap': }
```

##### Install the 'postgresql' key using application specific defaults

```puppet

openssl::key { 'postgresql':
  key     => $facts['networking']['hostname'],
  owner   => 'root',
  group   => 'postgres',
  mode    => '0440',
  key_dir => '/etc/postgresql',
  source  => $facts['networking']['hostname'],
}
```

#### Parameters

The following parameters are available in the `openssl::key` defined type:

* [`ensure`](#-openssl--key--ensure)
* [`key`](#-openssl--key--key)
* [`source`](#-openssl--key--source)
* [`extension`](#-openssl--key--extension)
* [`source_extension`](#-openssl--key--source_extension)
* [`mode`](#-openssl--key--mode)
* [`owner`](#-openssl--key--owner)
* [`group`](#-openssl--key--group)
* [`key_dir`](#-openssl--key--key_dir)

##### <a name="-openssl--key--ensure"></a>`ensure`

Data type: `Enum['present','absent']`

The state of the resource.

Default value: `'present'`

##### <a name="-openssl--key--key"></a>`key`

Data type: `String`

The basename of the file where the key will be stored on the client. The
full filename will be created using the three components `key_dir`, `key`
and `extension`.

Default value: `$name`

##### <a name="-openssl--key--source"></a>`source`

Data type: `String`

The basename of the file where the key is stored on the server. The full
filename will be created using the three parameters
`cert_source_directory` (see the base class `openssl`), `source` and
`source_extension`.

Default value: `$name`

##### <a name="-openssl--key--extension"></a>`extension`

Data type: `String`

The file extension used for files created on the client.

Default value: `'key'`

##### <a name="-openssl--key--source_extension"></a>`source_extension`

Data type: `String`

The file extension used for files read on the server.

Default value: `'key'`

##### <a name="-openssl--key--mode"></a>`mode`

Data type: `Stdlib::Filemode`

The file mode used for the resource.

Default value: `'0400'`

##### <a name="-openssl--key--owner"></a>`owner`

Data type: `String`

The file owner used for the resource.

Default value: `'root'`

##### <a name="-openssl--key--group"></a>`group`

Data type: `Optional[String]`

The file group used for the resource.

Default value: `undef`

##### <a name="-openssl--key--key_dir"></a>`key_dir`

Data type: `Optional[Stdlib::Absolutepath]`

The destination directory on the client where the key will be stored.

Default value: `undef`

## Resource types

### <a name="openssl_cert"></a>`openssl_cert`

The type takes a Certificate Signing Request (create by `openssl_request`
for example) and an issuer certificate and key as input to generate
a signed certificate.

To create a self-signed certificate, set `issuer_key` to the same key
that was used to create the request. Otherwise `issuer_cert` and
`issuer_key` should point to your CA certificate and key.

The type uses a random 128 bit number as serial number.

The certificate validity starts the moment the certificate is signed and
terminates as defined by the parameter `days`. The expiration time of the
cerificate is additionally limited by the validity of your CA certificate
unless you create a self-signed certificate.

The parameters `copy_request_extensions` and `omit_request_extensions`
can be used to specifically allow or deny some extensions from the
request. You can also use type parameters to set some extensions to
a fixed value.

The type expects to find the "-----BEGIN CERTIFICATE-----" token in the
file or it will overwrite the file content with a new certificate.

The type is refreshable and will generate a new certificate if the
resource is notified from another resource.

This type uses the Ruby OpenSSL library and does not need the `openssl`
binary provided by the operating system.

**Autorequires:** If Puppet is managing the OpenSSL issuer key, issuer
certificate or request that is used to create the certificate, the
`openssl_cert` resource will autorequire these resources

#### Examples

##### Create CA certificate from a CSR using the specified extensions

```puppet

openssl_cert { '/etc/ssl/ca.crt':
  request                       => '/etc/ssl/ca.csr',
  issuer_key                    => '/etc/ssl/ca.key',
  key_usage                     => ['keyCertSign', 'cRLSign'],
  key_usage_critical            => true,
  basic_constraints_ca          => true,
  basic_constraints_ca_critical => true,
  subject_key_identifier        => 'hash',
  authority_key_identifier      => ['issuer', 'keyid:always'],
  days                          => 2922,
}
```

##### Create certificate for a node and copy two extensions from the CSR

```puppet

openssl_cert { "/etc/ssl/${facts[networking][fqdn]}.crt":
  request                  => "/etc/ssl/${facts[networking][fqdn]}.csr",
  issuer_key               => '/etc/ssl/ca.key',
  issuer_cert              => '/etc/ssl/ca.crt',
  subject_key_identifier   => 'hash',
  authority_key_identifier => ['keyid', 'issuer'],
  copy_request_extensions  => ['subjectAltName', 'keyUsage'],
}
```

#### Properties

The following properties are available in the `openssl_cert` type.

##### `ensure`

Valid values: `present`, `absent`

The basic property that the resource should be in.

Default value: `present`

#### Parameters

The following parameters are available in the `openssl_cert` type.

* [`authority_key_identifier`](#-openssl_cert--authority_key_identifier)
* [`backup`](#-openssl_cert--backup)
* [`basic_constraints_ca`](#-openssl_cert--basic_constraints_ca)
* [`basic_constraints_ca_critical`](#-openssl_cert--basic_constraints_ca_critical)
* [`copy_request_extensions`](#-openssl_cert--copy_request_extensions)
* [`days`](#-openssl_cert--days)
* [`extended_key_usage`](#-openssl_cert--extended_key_usage)
* [`extended_key_usage_critical`](#-openssl_cert--extended_key_usage_critical)
* [`group`](#-openssl_cert--group)
* [`issuer_cert`](#-openssl_cert--issuer_cert)
* [`issuer_key`](#-openssl_cert--issuer_key)
* [`issuer_key_password`](#-openssl_cert--issuer_key_password)
* [`key_usage`](#-openssl_cert--key_usage)
* [`key_usage_critical`](#-openssl_cert--key_usage_critical)
* [`mode`](#-openssl_cert--mode)
* [`omit_request_extensions`](#-openssl_cert--omit_request_extensions)
* [`owner`](#-openssl_cert--owner)
* [`path`](#-openssl_cert--path)
* [`request`](#-openssl_cert--request)
* [`selinux_ignore_defaults`](#-openssl_cert--selinux_ignore_defaults)
* [`selrange`](#-openssl_cert--selrange)
* [`selrole`](#-openssl_cert--selrole)
* [`seltype`](#-openssl_cert--seltype)
* [`seluser`](#-openssl_cert--seluser)
* [`show_diff`](#-openssl_cert--show_diff)
* [`signature_algorithm`](#-openssl_cert--signature_algorithm)
* [`subject_key_identifier`](#-openssl_cert--subject_key_identifier)
* [`subject_key_identifier_critical`](#-openssl_cert--subject_key_identifier_critical)

##### <a name="-openssl_cert--authority_key_identifier"></a>`authority_key_identifier`

The Authority Key Identifier extension.

##### <a name="-openssl_cert--backup"></a>`backup`

Specifies whether (and how) to back up the destination file before
overwriting it. Your value gets passed on to Puppet's native file
resource for execution. Valid options: true, false, or a string
representing either a target filebucket or a filename extension
beginning with ".".

##### <a name="-openssl_cert--basic_constraints_ca"></a>`basic_constraints_ca`

Valid values: `true`, `false`

Whether the Basic Constraints CA extension should be set.

Setting this parameter overrides the value of the `basicConstraints`
extension from the request.

##### <a name="-openssl_cert--basic_constraints_ca_critical"></a>`basic_constraints_ca_critical`

Valid values: `true`, `false`

Whether the Basic Constraints CA extension should be critical.

##### <a name="-openssl_cert--copy_request_extensions"></a>`copy_request_extensions`

List of extensions to copy from the certificate request. If this
parameter is set, then only these extensions are copied from the
request into the final certificate. Otherwise all extensions are copied
from the request unless the parameter `omit_request_extensions`
disallows them.

Some extension names that might be useful to include here are
`basicConstraints`, `keyUsage`, `extendedKeyUsage`, `subjectAltName`.

If an extension name is included in `copy_request_extension` and
`omit_request_extensions`, then `omit_request_extensions` has
precedence and the extension is not copied from the request to the
final certificate.

Extensions defined by explicit type parameters always override
extensions from the request.

Default value: `[]`

##### <a name="-openssl_cert--days"></a>`days`

Valid values: `%r{^[0-9]+$}`

The number of days that the certificate should be valid.

A certificate can't be valid after the issuing certificate has
expired. So the validity is limited by the expiration time of the
issuing certificate.

Default value: `365`

##### <a name="-openssl_cert--extended_key_usage"></a>`extended_key_usage`

The X.509v3 Extended Key Usage extension. Valid options: `serverAuth`,
`clientAuth`, `codeSigning`, `emailProtection`, `timeStamping`,
`OCSPSigning`, `ipsecIKE`, `msCodeInd`, `msCodeCom`, `msCTLSign`,
`msEFS`.

Setting this parameter overrides the value of the `extendedKeyUsage`
extension from the request.

##### <a name="-openssl_cert--extended_key_usage_critical"></a>`extended_key_usage_critical`

Valid values: `true`, `false`

Whether the Extenden Key Usage extension should be critical.

##### <a name="-openssl_cert--group"></a>`group`

Specifies a permissions group for the destination file. Valid options:
a string containing a group name or integer containing a gid.

##### <a name="-openssl_cert--issuer_cert"></a>`issuer_cert`

The path to the certificate file that is used to issue the certificate.

##### <a name="-openssl_cert--issuer_key"></a>`issuer_key`

The path to the key file that is used to issue the certificate. If this
is the same key that was used to create the request, then a self-signed
certificate will be created.

##### <a name="-openssl_cert--issuer_key_password"></a>`issuer_key_password`

The password to use when loading a protected issuer key.

Default value: `''`

##### <a name="-openssl_cert--key_usage"></a>`key_usage`

The X.509v3 Key Usage extension. Valid options: `digitalSignature`,
`nonRepudiation`, `keyEncipherment`, `dataEncipherment`,
`keyAgreement`, `keyCertSign`, `cRLSign`, `encipherOnly`,
`decipherOnly`.

Setting this parameter overrides the value of the `keyUsage` extension
from the request.

##### <a name="-openssl_cert--key_usage_critical"></a>`key_usage_critical`

Valid values: `true`, `false`

Whether the Key Usage extension should be critical.

##### <a name="-openssl_cert--mode"></a>`mode`

Specifies the permissions mode of the destination file. Valid options:
a string containing a permission mode value in octal notation.

##### <a name="-openssl_cert--omit_request_extensions"></a>`omit_request_extensions`

List of extensions to omit from the certificate request. If this
parameter is set, then the named extensions are never copied from the
request into the final certificate. Otherwise all extensions are copied
from the request unless the parameter `copy_request_extensions`
restricts them.

Some extension names that might be useful to include here are
`basicConstraints`, `keyUsage`, `extendedKeyUsage`, `subjectAltName`.

If an extension name is include in `copy_request_extension` and
`omit_request_extensions`, then `omit_request_extensions` has
precedence and the extension is not copied from the request to the
final certificate.

Extensions defined by explicit type parameters always override
extensions from the request.

Default value: `[]`

##### <a name="-openssl_cert--owner"></a>`owner`

Specifies the owner of the destination file. Valid options: a string
containing a username or integer containing a uid.

##### <a name="-openssl_cert--path"></a>`path`

Specifies the destination file. Valid options: a string containing an
absolute path. Default value: the title of your declared resource.

##### <a name="-openssl_cert--request"></a>`request`

The path to the certificate request to use when creating the certificate.

##### <a name="-openssl_cert--selinux_ignore_defaults"></a>`selinux_ignore_defaults`

Valid values: `true`, `false`, `yes`, `no`

See the file type's selinux_ignore_defaults documentention:
https://docs.puppetlabs.com/references/latest/type.html#file-attribute-selinux_ignore_defaults.

##### <a name="-openssl_cert--selrange"></a>`selrange`

See the file type's selrange documentation:
https://docs.puppetlabs.com/references/latest/type.html#file-attribute-selrange

##### <a name="-openssl_cert--selrole"></a>`selrole`

See the file type's selrole documentation:
https://docs.puppetlabs.com/references/latest/type.html#file-attribute-selrole

##### <a name="-openssl_cert--seltype"></a>`seltype`

See the file type's seltype documentation:
https://docs.puppetlabs.com/references/latest/type.html#file-attribute-seltype

##### <a name="-openssl_cert--seluser"></a>`seluser`

See the file type's seluser documentation:
https://docs.puppetlabs.com/references/latest/type.html#file-attribute-seluser

##### <a name="-openssl_cert--show_diff"></a>`show_diff`

Valid values: `true`, `false`, `yes`, `no`

Specifies whether to set the show_diff parameter for the file
resource.

##### <a name="-openssl_cert--signature_algorithm"></a>`signature_algorithm`

Valid values: `md2`, `md4`, `md5`, `sha`, `sha1`, `sha224`, `sha256`, `sha384`, `sha512`

The signature algorithm to use. The algorithms `md2`, `md4`, `md5`,
`sha` and `sha1` are only included for backwards compatibility and
should be considered insecure for new certificates.

Default value: `sha256`

##### <a name="-openssl_cert--subject_key_identifier"></a>`subject_key_identifier`

The Subject Key Identifier extension. Normally the value `hash` is used
when creating certificates.

##### <a name="-openssl_cert--subject_key_identifier_critical"></a>`subject_key_identifier_critical`

Valid values: `true`, `false`

Whether the Subject Key Identifier extension should be critical.

### <a name="openssl_certutil"></a>`openssl_certutil`

This type installs the certificate specified with `filename` as a trusted
certificate if `ensure => present`. The trust is removed if `ensure =>
absent`.

The `certutil` executable is required for this type. In general it is
only available on RedHat-based distributions.

The certificate file itself is not managed by this type.

The file must already exist on the node before it can be added to the NSS
database. Make sure you add the correct dependency if you manage the
certificate file with Puppet.

There is an unsolved issue if a certificate is added a second time to the
NSS database using a different name. In this case `certutil` does not add
the certificate but also does not report an error. Therefore Puppet will
try to add the certificate every time it runs. As a workaround the
already installed certificate should be removed.

#### Examples

##### Add a certificate to the NSS database and set trust level for SSL

```puppet

openssl_certutil { '/etc/ssl/certs/My-Root-CA.crt':
  ensure    => present,
  ssl_trust => 'C',
}
```

##### Remove a certificate from the NSS database

```puppet

openssl_certutil { '/etc/ssl/certs/My-Root-CA.crt':
  ensure => absent,
}
```

#### Properties

The following properties are available in the `openssl_certutil` type.

##### `email_trust`

Valid values: `%r{[pPcCT]*}`

Email trust attributes for the certificate.

##### `ensure`

Valid values: `present`, `absent`

The basic property that the resource should be in.

Default value: `present`

##### `object_signing_trust`

Valid values: `%r{[pPcCT]*}`

Object signing trust attributes for the certificate.

##### `ssl_trust`

Valid values: `%r{[pPcCT]*}`

SSL trust attributes for the certificate.

#### Parameters

The following parameters are available in the `openssl_certutil` type.

* [`filename`](#-openssl_certutil--filename)
* [`name`](#-openssl_certutil--name)
* [`provider`](#-openssl_certutil--provider)

##### <a name="-openssl_certutil--filename"></a>`filename`

The filename of the certificate.

##### <a name="-openssl_certutil--name"></a>`name`

namevar

The nickname of the certificate in the certificate database.

##### <a name="-openssl_certutil--provider"></a>`provider`

The specific backend to use for this `openssl_certutil` resource. You will seldom need to specify this --- Puppet will
usually discover the appropriate provider for your platform.

### <a name="openssl_dhparam"></a>`openssl_dhparam`

Generate Diffie-Hellman parameters for an TLS enabled application by
specifying the number of bits and the generator number to use.

The type expects to find the "-----BEGIN DH PARAMETERS-----" token in the
file or it will overwrite the file content with new parameters.

The type is refreshable and will generate new parameters if the resource
is notified from another resource.

This type uses the Ruby OpenSSL library and does not need the `openssl`
binary provided by the operating system.

*Note*: The creation of Diffie-Hellman parameters with a larger number of
bits takes a significant amount of CPU time (sometimes multiple
minutes). This might look as if the Puppet Agent is hanging.

#### Examples

##### Generate Diffie-Hellman parameter file

```puppet

openssl_dhparam { '/etc/postfix/dh2048.pem':
  owner   => 'root',
  group   => 'root',
  mode    => '0644'
  require => Package['postfix'],
  notify  => Service['postfix'],
}
```

##### Trigger refresh using another resource

```puppet

openssl_dhparam { '/etc/postfix/dh2048.pem':
  subscribe => Package['postfix'],
}
```

#### Properties

The following properties are available in the `openssl_dhparam` type.

##### `ensure`

Valid values: `present`, `absent`

The basic property that the resource should be in.

Default value: `present`

#### Parameters

The following parameters are available in the `openssl_dhparam` type.

* [`backup`](#-openssl_dhparam--backup)
* [`bits`](#-openssl_dhparam--bits)
* [`generator`](#-openssl_dhparam--generator)
* [`group`](#-openssl_dhparam--group)
* [`mode`](#-openssl_dhparam--mode)
* [`owner`](#-openssl_dhparam--owner)
* [`path`](#-openssl_dhparam--path)
* [`selinux_ignore_defaults`](#-openssl_dhparam--selinux_ignore_defaults)
* [`selrange`](#-openssl_dhparam--selrange)
* [`selrole`](#-openssl_dhparam--selrole)
* [`seltype`](#-openssl_dhparam--seltype)
* [`seluser`](#-openssl_dhparam--seluser)
* [`show_diff`](#-openssl_dhparam--show_diff)

##### <a name="-openssl_dhparam--backup"></a>`backup`

Specifies whether (and how) to back up the destination file before
overwriting it. Your value gets passed on to Puppet's native file
resource for execution. Valid options: true, false, or a string
representing either a target filebucket or a filename extension
beginning with ".".

##### <a name="-openssl_dhparam--bits"></a>`bits`

Valid values: `1024`, `2048`, `3072`, `4096`, `5120`, `6144`, `7168`, `8192`

The number of bits for the Diffie-Hellman parameters.

Default value: `2048`

##### <a name="-openssl_dhparam--generator"></a>`generator`

Valid values: `2`, `5`

The generator number for the Diffie-Hellman parameters.

Default value: `2`

##### <a name="-openssl_dhparam--group"></a>`group`

Specifies a permissions group for the destination file. Valid options:
a string containing a group name or integer containing a gid.

##### <a name="-openssl_dhparam--mode"></a>`mode`

Specifies the permissions mode of the destination file. Valid options:
a string containing a permission mode value in octal notation.

##### <a name="-openssl_dhparam--owner"></a>`owner`

Specifies the owner of the destination file. Valid options: a string
containing a username or integer containing a uid.

##### <a name="-openssl_dhparam--path"></a>`path`

Specifies the destination file. Valid options: a string containing an
absolute path. Default value: the title of your declared resource.

##### <a name="-openssl_dhparam--selinux_ignore_defaults"></a>`selinux_ignore_defaults`

Valid values: `true`, `false`, `yes`, `no`

See the file type's selinux_ignore_defaults documentention:
https://docs.puppetlabs.com/references/latest/type.html#file-attribute-selinux_ignore_defaults.

##### <a name="-openssl_dhparam--selrange"></a>`selrange`

See the file type's selrange documentation:
https://docs.puppetlabs.com/references/latest/type.html#file-attribute-selrange

##### <a name="-openssl_dhparam--selrole"></a>`selrole`

See the file type's selrole documentation:
https://docs.puppetlabs.com/references/latest/type.html#file-attribute-selrole

##### <a name="-openssl_dhparam--seltype"></a>`seltype`

See the file type's seltype documentation:
https://docs.puppetlabs.com/references/latest/type.html#file-attribute-seltype

##### <a name="-openssl_dhparam--seluser"></a>`seluser`

See the file type's seluser documentation:
https://docs.puppetlabs.com/references/latest/type.html#file-attribute-seluser

##### <a name="-openssl_dhparam--show_diff"></a>`show_diff`

Valid values: `true`, `false`, `yes`, `no`

Specifies whether to set the show_diff parameter for the file
resource.

### <a name="openssl_genparam"></a>`openssl_genparam`

The type is refreshable. The `openssl_genparam` type will regenerate the
parameters if the resource is notified from another resource.

#### Examples

##### Create a Diffie-Hellman parameter file using 2048 bits

```puppet

openssl_genparam { '/tmp/dhparam.pem':
  algorithm => 'DH',
  bits      => '2048,
  generator => '2',
}
```

##### Create an Elliptic Curve parameter file using the secp521e1 curve

```puppet

openssl_genparam { '/tmp/ecparam.pem':
  algorithm => 'EC',
  curve     => 'secp521r1',
}
```

##### Automatically refresh a parameter file every 3 months

```puppet

openssl_genparam { '/tmp/dhparam.pem':
  algorithm        => 'DH',
  bits             => '2048,
  generator        => '2',
  refresh_interval => '3mo',
}
```

##### Refresh a parameter file if another file changes

```puppet

openssl_genparam { '/tmp/dhparam.pem':
  algorithm => 'DH',
  bits      => '2048,
  subscribe => File['/etc/ssl/parameters.trigger'],
}
```

#### Properties

The following properties are available in the `openssl_genparam` type.

##### `ensure`

Valid values: `present`, `absent`

The basic property that the resource should be in.

Default value: `present`

#### Parameters

The following parameters are available in the `openssl_genparam` type.

* [`algorithm`](#-openssl_genparam--algorithm)
* [`bits`](#-openssl_genparam--bits)
* [`curve`](#-openssl_genparam--curve)
* [`file`](#-openssl_genparam--file)
* [`generator`](#-openssl_genparam--generator)
* [`provider`](#-openssl_genparam--provider)
* [`refresh_interval`](#-openssl_genparam--refresh_interval)

##### <a name="-openssl_genparam--algorithm"></a>`algorithm`

Valid values: `DH`, `EC`

The algorithm to generate the parameters for.

##### <a name="-openssl_genparam--bits"></a>`bits`

Valid values: `2048`, `4096`, `8192`

The number of bits to use for Diffie-Hellman parameters.

##### <a name="-openssl_genparam--curve"></a>`curve`

Valid values: `%r{^[a-zA-Z][a-zA-Z0-9-]+[0-9]$}`

The name of the curve to use for Elliptic Curve parameters.

##### <a name="-openssl_genparam--file"></a>`file`

The name of the parameter file to manage.

##### <a name="-openssl_genparam--generator"></a>`generator`

Valid values: `2`, `5`

The generator to use for Diffie-Hellman parameters.

##### <a name="-openssl_genparam--provider"></a>`provider`

The specific backend to use for this `openssl_genparam` resource. You will seldom need to specify this --- Puppet will
usually discover the appropriate provider for your platform.

##### <a name="-openssl_genparam--refresh_interval"></a>`refresh_interval`

Valid values: `%r{^[0-9]+(y|mo|w|d|h|mi|s)?$}`

The Refresh interval for the parameter file. A new parameter file
will be generated after this time.

The value must be a number optionally followed by a time unit. The
following units are understood: `y` for year (365 days), `mo` for
months (30 days), `w` for week (7 days), `d` for days (24 hours), `h`
for hours (60 minutes), `mi` for minute (60 seconds). When the unit `s`
or no unit is used then the value is interpreted as the number of
seconds.

### <a name="openssl_genpkey"></a>`openssl_genpkey`

**This type is deprecated!**

Generate an OpenSSL private key file. The type creates RSA or Elliptic
Curve keys depending on the parameter `algorithm`.

The key can optionally be encrypted using a supplied password. Encryption
uses the `-passin` option when calling `openssl` so the password is not
visible in the process listing.

The type is refreshable. The `openssl_genpkey` type will regenerate the
key if the resource is notified from another resource.

#### Examples

##### Generate a 2048 bit RSA key file

```puppet

openssl_genpkey { '/tmp/rsa-2048.key':
  algorithm => 'RSA',
  bits      => '2048',
}
```

##### Generate AES encrypted Elliptic Curve private key

```puppet

openssl_genpkey { '/tmp/ec-secp256k1.key':
  algorithm => 'EC',
  curve     => 'secp256k1',
  cipher    => 'aes128',
  password  => 'rosebud',
}
```

##### Regenerate the key if another file changes

```puppet

openssl_genpkey { '/tmp/rsa-2048.key':
  algorithm => 'RSA',
  bits      => '2048',
  subscribe => File['/etc/ssl/key.trigger'],
}
```

#### Properties

The following properties are available in the `openssl_genpkey` type.

##### `ensure`

Valid values: `present`, `absent`

The basic property that the resource should be in.

Default value: `present`

#### Parameters

The following parameters are available in the `openssl_genpkey` type.

* [`algorithm`](#-openssl_genpkey--algorithm)
* [`bits`](#-openssl_genpkey--bits)
* [`cipher`](#-openssl_genpkey--cipher)
* [`curve`](#-openssl_genpkey--curve)
* [`file`](#-openssl_genpkey--file)
* [`password`](#-openssl_genpkey--password)
* [`provider`](#-openssl_genpkey--provider)

##### <a name="-openssl_genpkey--algorithm"></a>`algorithm`

Valid values: `RSA`, `EC`

The algorithm to generate a private key for. The number of bits
must be supplied if an RSA key is generated. For an EC key the curve
name must be given

##### <a name="-openssl_genpkey--bits"></a>`bits`

Valid values: `2048`, `3072`, `4096`, `5120`, `6144`, `7168`, `8192`

The number of bits for the RSA key. This parameter is mandatory for
RSA keys.

##### <a name="-openssl_genpkey--cipher"></a>`cipher`

Encrypt the key with the supplied cipher. A password must be given
in this case.

##### <a name="-openssl_genpkey--curve"></a>`curve`

Valid values: `%r{^[a-zA-Z][a-zA-Z0-9-]+[0-9]$}`

The curve to use for elliptic curve key. This parameter is
mandatory for EC keys.

##### <a name="-openssl_genpkey--file"></a>`file`

The name of the private key file to manage.

##### <a name="-openssl_genpkey--password"></a>`password`

Use the supplied password when encrypting the key.

##### <a name="-openssl_genpkey--provider"></a>`provider`

The specific backend to use for this `openssl_genpkey` resource. You will seldom need to specify this --- Puppet will
usually discover the appropriate provider for your platform.

### <a name="openssl_hash"></a>`openssl_hash`

If `ensure => present` a symbolic link using the certificate hash will be
created in the same directory as the certificate. The link is removed if
`ensure => absent`.

This link is used to find a trusted cert when a certificate chain is
validated.

The certificate file itself is not managed by this type.

The file must exist before the link can be created as it is accessed by
OpenSSL to calculate the hash. For the same reason the file can only be
deleted after the link has been removed.

#### Examples

##### Mark an existing certificate as trusted

```puppet

openssl_trustcert { '/etc/ssl/certs/My-Root-CA.crt':
  ensure => present,
}
```

##### Mark an existing certificate as not trusted

```puppet

openssl_trustcert { '/etc/ssl/certs/My-Root-CA.crt':
  ensure => absent,
}
```

#### Properties

The following properties are available in the `openssl_hash` type.

##### `ensure`

Valid values: `present`, `absent`

The basic property that the resource should be in.

Default value: `present`

#### Parameters

The following parameters are available in the `openssl_hash` type.

* [`name`](#-openssl_hash--name)
* [`provider`](#-openssl_hash--provider)

##### <a name="-openssl_hash--name"></a>`name`

namevar

The name of the certificate file to manage.

##### <a name="-openssl_hash--provider"></a>`provider`

The specific backend to use for this `openssl_hash` resource. You will seldom need to specify this --- Puppet will
usually discover the appropriate provider for your platform.

### <a name="openssl_key"></a>`openssl_key`

This type creates RSA or Elliptic Curve keys depending on the parameter
`algorithm`.

The key can optionally be encrypted using a supplied password.

The type expects to find the "-----BEGIN PRIVATE KEY-----" token in the
file or it will overwrite the file content with a new key.

The type is refreshable and will generate a new key if the resource is
notified from another resource.

This type uses the Ruby OpenSSL library and does not need the `openssl`
binary provided by the operating system.

#### Examples

##### Generate a 2048 bit RSA key

```puppet

openssl_key { '/etc/ssl/rsa-2048.key':
  algorithm => 'RSA',
  bits      => 2048,
}
```

##### Generate an Elliptic Curve key that is encrypted using AES128

```puppet

openssl_key { '/etc/ssl/ec-secp256k1.key':
  algorithm => 'EC',
  curve     => 'secp256k1',
  cipher    => 'aes128',
  password  => 'rosebud',
}
```

##### Create a key and regenerate it if another resource changes

```puppet

openssl_key { '/etc/ssl/rsa-2048.key':
  algorithm => 'RSA',
  bits      => 2048,
  subscribe => File['/etc/ssl/key.trigger'],
}
```

#### Properties

The following properties are available in the `openssl_key` type.

##### `ensure`

Valid values: `present`, `absent`

The basic property that the resource should be in.

Default value: `present`

#### Parameters

The following parameters are available in the `openssl_key` type.

* [`algorithm`](#-openssl_key--algorithm)
* [`backup`](#-openssl_key--backup)
* [`bits`](#-openssl_key--bits)
* [`cipher`](#-openssl_key--cipher)
* [`curve`](#-openssl_key--curve)
* [`group`](#-openssl_key--group)
* [`mode`](#-openssl_key--mode)
* [`owner`](#-openssl_key--owner)
* [`password`](#-openssl_key--password)
* [`path`](#-openssl_key--path)
* [`selinux_ignore_defaults`](#-openssl_key--selinux_ignore_defaults)
* [`selrange`](#-openssl_key--selrange)
* [`selrole`](#-openssl_key--selrole)
* [`seltype`](#-openssl_key--seltype)
* [`seluser`](#-openssl_key--seluser)
* [`show_diff`](#-openssl_key--show_diff)

##### <a name="-openssl_key--algorithm"></a>`algorithm`

Valid values: `RSA`, `EC`

The algorithm to use when generating a private key. The number of bits
must be supplied if an RSA key is generated. For an EC key the curve
name must be given.

Default value: `RSA`

##### <a name="-openssl_key--backup"></a>`backup`

Specifies whether (and how) to back up the destination file before
overwriting it. Your value gets passed on to Puppet's native file
resource for execution. Valid options: true, false, or a string
representing either a target filebucket or a filename extension
beginning with ".".

##### <a name="-openssl_key--bits"></a>`bits`

Valid values: `1024`, `2048`, `3072`, `4096`, `5120`, `6144`, `7168`, `8192`

The number of bits for the RSA key. This parameter is mandatory for RSA
keys. Keys with 1024 bits should only be used for specific applications
like DKIM.

Default value: `2048`

##### <a name="-openssl_key--cipher"></a>`cipher`

Encrypt the key with the supplied cipher. A password must be given if
this parameter is set.

##### <a name="-openssl_key--curve"></a>`curve`

The curve to use for elliptic curve key. This parameter is mandatory
for EC keys. Consult your OpenSSL documentation to find out what curves
are supported on your system. The following curves should be available
for TLS 1.3 and earlier: `secp256r1`, `secp384r1`, `secp521r1`.

Default value: `secp384r1`

##### <a name="-openssl_key--group"></a>`group`

Specifies a permissions group for the destination file. Valid options:
a string containing a group name or integer containing a gid.

##### <a name="-openssl_key--mode"></a>`mode`

Specifies the permissions mode of the destination file. Valid options:
a string containing a permission mode value in octal notation.

##### <a name="-openssl_key--owner"></a>`owner`

Specifies the owner of the destination file. Valid options: a string
containing a username or integer containing a uid.

##### <a name="-openssl_key--password"></a>`password`

Use the supplied password to encrypt the key. Setting only a password
without a cipher creates an unprotected key.

##### <a name="-openssl_key--path"></a>`path`

Specifies the destination file. Valid options: a string containing an
absolute path. Default value: the title of your declared resource.

##### <a name="-openssl_key--selinux_ignore_defaults"></a>`selinux_ignore_defaults`

Valid values: `true`, `false`, `yes`, `no`

See the file type's selinux_ignore_defaults documentention:
https://docs.puppetlabs.com/references/latest/type.html#file-attribute-selinux_ignore_defaults.

##### <a name="-openssl_key--selrange"></a>`selrange`

See the file type's selrange documentation:
https://docs.puppetlabs.com/references/latest/type.html#file-attribute-selrange

##### <a name="-openssl_key--selrole"></a>`selrole`

See the file type's selrole documentation:
https://docs.puppetlabs.com/references/latest/type.html#file-attribute-selrole

##### <a name="-openssl_key--seltype"></a>`seltype`

See the file type's seltype documentation:
https://docs.puppetlabs.com/references/latest/type.html#file-attribute-seltype

##### <a name="-openssl_key--seluser"></a>`seluser`

See the file type's seluser documentation:
https://docs.puppetlabs.com/references/latest/type.html#file-attribute-seluser

##### <a name="-openssl_key--show_diff"></a>`show_diff`

Valid values: `true`, `false`, `yes`, `no`

Specifies whether to set the show_diff parameter for the file
resource.

### <a name="openssl_request"></a>`openssl_request`

The type creates a X.509 Certificate Signing Request (CSR) which can
either be submitted to a Certificate Authority (CA) for signing or used
to create a self-signed certificate. Both operations can also be
performed using the `openssl_cert` type.

The X.509 subject of the request can be defined by using the
`common_name`, `domain_component`, `organization_unit_name`,
`organization_name`, `locality_name`, `state_or_province_name`,
`country_name` and `email_address` parameters. Setting a Common Name is
mandatory and the host fully-qualified domain name (FQDN) is commonly
used for node or service certificates.

The request can also include the following extensions by setting the
appropriate type parameters: `basicConstraints`, `keyUsage`,
`extendedKeyUsage` and `subjectAltName`.

The type expects to find the "-----BEGIN CERTIFICATE REQUEST-----" token
in the file or it will overwrite the file content with a new request.

The type is refreshable and will generate a new request if the resource
is notified from another resource.

This type uses the Ruby OpenSSL library and does not need the `openssl`
binary provided by the operating system.

**Autorequires:** If Puppet is managing the OpenSSL key that is used to
create the CSR, the `openssl_request` resource will autorequire that key.

#### Examples

##### Generate CSR to be used for a private Certificate Authority

```puppet

openssl_request { '/etc/ssl/ca.csr':
  key              => '/etc/ssl/ca.key',
  common_name      => 'ACME Root CA',
  domain_component => [ 'ACME', 'US' ],
}
```

##### Generate CSR for a web application

```puppet

openssl_request { "/etc/ssl/app.example.com.csr":
  key                         => '/etc/ssl/app.example.com.key',
  common_name                 => 'app.example.com',
  key_usage                   => ['keyEncipherment', 'digitalSignature'],
  extended_key_usage          => ['serverAuth', 'clientAuth'],
  subject_alternate_names_dns => ['app.example.com'],
  subject_alternate_names_ip  => ['192.0.2.42'],
}
```

#### Properties

The following properties are available in the `openssl_request` type.

##### `ensure`

Valid values: `present`, `absent`

The basic property that the resource should be in.

Default value: `present`

#### Parameters

The following parameters are available in the `openssl_request` type.

* [`authenticator_control`](#-openssl_request--authenticator_control)
* [`backup`](#-openssl_request--backup)
* [`basic_constraints_ca`](#-openssl_request--basic_constraints_ca)
* [`basic_constraints_ca_critical`](#-openssl_request--basic_constraints_ca_critical)
* [`common_name`](#-openssl_request--common_name)
* [`country_name`](#-openssl_request--country_name)
* [`domain_component`](#-openssl_request--domain_component)
* [`email_address`](#-openssl_request--email_address)
* [`extended_key_usage`](#-openssl_request--extended_key_usage)
* [`extended_key_usage_critical`](#-openssl_request--extended_key_usage_critical)
* [`group`](#-openssl_request--group)
* [`key`](#-openssl_request--key)
* [`key_password`](#-openssl_request--key_password)
* [`key_usage`](#-openssl_request--key_usage)
* [`key_usage_critical`](#-openssl_request--key_usage_critical)
* [`locality_name`](#-openssl_request--locality_name)
* [`mode`](#-openssl_request--mode)
* [`organization_name`](#-openssl_request--organization_name)
* [`organization_unit_name`](#-openssl_request--organization_unit_name)
* [`owner`](#-openssl_request--owner)
* [`path`](#-openssl_request--path)
* [`registration_token_control`](#-openssl_request--registration_token_control)
* [`selinux_ignore_defaults`](#-openssl_request--selinux_ignore_defaults)
* [`selrange`](#-openssl_request--selrange)
* [`selrole`](#-openssl_request--selrole)
* [`seltype`](#-openssl_request--seltype)
* [`seluser`](#-openssl_request--seluser)
* [`serial`](#-openssl_request--serial)
* [`show_diff`](#-openssl_request--show_diff)
* [`signature_algorithm`](#-openssl_request--signature_algorithm)
* [`state_or_province_name`](#-openssl_request--state_or_province_name)
* [`subject_alternate_names_dns`](#-openssl_request--subject_alternate_names_dns)
* [`subject_alternate_names_ip`](#-openssl_request--subject_alternate_names_ip)

##### <a name="-openssl_request--authenticator_control"></a>`authenticator_control`

An authenticator control contains information used to establish
a non-cryptographic check of identity by the CA. Consult the
documentation of your CA if this is necessary and what value you need
to set. Also see RFC RFC 4211 for more details.

##### <a name="-openssl_request--backup"></a>`backup`

Specifies whether (and how) to back up the destination file before
overwriting it. Your value gets passed on to Puppet's native file
resource for execution. Valid options: true, false, or a string
representing either a target filebucket or a filename extension
beginning with ".".

##### <a name="-openssl_request--basic_constraints_ca"></a>`basic_constraints_ca`

Valid values: `true`, `false`, `yes`, `no`

Whether the Basic Constraints CA extension should be set.

##### <a name="-openssl_request--basic_constraints_ca_critical"></a>`basic_constraints_ca_critical`

Valid values: `true`, `false`, `yes`, `no`

Whether the Basic Constraints CA extension should be critical.

##### <a name="-openssl_request--common_name"></a>`common_name`

The value of the X.509 common name (CN) attribute.

##### <a name="-openssl_request--country_name"></a>`country_name`

The value of the X.509 country (C) attribute.

##### <a name="-openssl_request--domain_component"></a>`domain_component`

The value of the X.509 domain component (DC) attributes. The value
should be an array. The items are used in the same order, so for
example the value `['example', 'com']` should be used to create
the attribute `DC=example, DC=com` in the request.

##### <a name="-openssl_request--email_address"></a>`email_address`

The value of the X.509 emailAddress attribute.

##### <a name="-openssl_request--extended_key_usage"></a>`extended_key_usage`

The X.509v3 Extended Key Usage extension.

##### <a name="-openssl_request--extended_key_usage_critical"></a>`extended_key_usage_critical`

Valid values: `true`, `false`, `yes`, `no`

Whether the Extenden Key Usage extension should be critical.

##### <a name="-openssl_request--group"></a>`group`

Specifies a permissions group for the destination file. Valid options:
a string containing a group name or integer containing a gid.

##### <a name="-openssl_request--key"></a>`key`

The path to the key file to use when creating the certificate request.

##### <a name="-openssl_request--key_password"></a>`key_password`

The password to use when loading a protected key.

Default value: `''`

##### <a name="-openssl_request--key_usage"></a>`key_usage`

The X.509v3 Key Usage extension.

##### <a name="-openssl_request--key_usage_critical"></a>`key_usage_critical`

Valid values: `true`, `false`, `yes`, `no`

Whether the Key Usage extension should be critical.

##### <a name="-openssl_request--locality_name"></a>`locality_name`

The value of the X.509 locality name (L) attribute.

##### <a name="-openssl_request--mode"></a>`mode`

Specifies the permissions mode of the destination file. Valid options:
a string containing a permission mode value in octal notation.

##### <a name="-openssl_request--organization_name"></a>`organization_name`

The value of the X.509 organization name (O) attribute.

##### <a name="-openssl_request--organization_unit_name"></a>`organization_unit_name`

The value of the X.509 organization unit name (OU) attribute.

##### <a name="-openssl_request--owner"></a>`owner`

Specifies the owner of the destination file. Valid options: a string
containing a username or integer containing a uid.

##### <a name="-openssl_request--path"></a>`path`

Specifies the destination file. Valid options: a string containing an
absolute path. Default value: the title of your declared resource.

##### <a name="-openssl_request--registration_token_control"></a>`registration_token_control`

The registration token control contains one-time information that can
be used by the CA to verify the identity of the subject. Consult the
documentation of your CA if this is necessary and what value you need
to set. Also see RFC RFC 4211 for more details.

##### <a name="-openssl_request--selinux_ignore_defaults"></a>`selinux_ignore_defaults`

Valid values: `true`, `false`, `yes`, `no`

See the file type's selinux_ignore_defaults documentention:
https://docs.puppetlabs.com/references/latest/type.html#file-attribute-selinux_ignore_defaults.

##### <a name="-openssl_request--selrange"></a>`selrange`

See the file type's selrange documentation:
https://docs.puppetlabs.com/references/latest/type.html#file-attribute-selrange

##### <a name="-openssl_request--selrole"></a>`selrole`

See the file type's selrole documentation:
https://docs.puppetlabs.com/references/latest/type.html#file-attribute-selrole

##### <a name="-openssl_request--seltype"></a>`seltype`

See the file type's seltype documentation:
https://docs.puppetlabs.com/references/latest/type.html#file-attribute-seltype

##### <a name="-openssl_request--seluser"></a>`seluser`

See the file type's seluser documentation:
https://docs.puppetlabs.com/references/latest/type.html#file-attribute-seluser

##### <a name="-openssl_request--serial"></a>`serial`

Valid values: `%r{^[0-9]+$}`

An otherwise unused serial number attribute that will be added to the
request. This can be useful to ensure that requests using the same key
and attributes (e.g. when regenerating the request after some time)
will give a different binary representation of the request and actually
trigger a refresh.

##### <a name="-openssl_request--show_diff"></a>`show_diff`

Valid values: `true`, `false`, `yes`, `no`

Specifies whether to set the show_diff parameter for the file
resource.

##### <a name="-openssl_request--signature_algorithm"></a>`signature_algorithm`

Valid values: `md2`, `md4`, `md5`, `sha`, `sha1`, `sha224`, `sha256`, `sha384`, `sha512`

The signature algorithm to use. The algorithms `md2`, `md4`, `md5`,
`sha` and `sha1` are only included for backwards compatibility and
should be considered insecure for new certificates.

Default value: `sha256`

##### <a name="-openssl_request--state_or_province_name"></a>`state_or_province_name`

The value of the X.509 state or province name (ST) attribute.

##### <a name="-openssl_request--subject_alternate_names_dns"></a>`subject_alternate_names_dns`

An array of DNS names that will be added as subject alternate names.

##### <a name="-openssl_request--subject_alternate_names_ip"></a>`subject_alternate_names_ip`

An array of IP addresses that will be added as subject alternate names.

### <a name="openssl_selfsign"></a>`openssl_selfsign`

**This type is deprecated!**

The type takes a certificate signing request (CSR) and a key file to
generate a self-signed certificate.

Certificate extensions can be added by using the `extensions` and
`extfile` parameters.

Optionally a key password can be provided if the used key is encrypted.

The certificate will be valid for the given number of days.

The type is refreshable. The `openssl_selfsign` type will regenerate the
certificate if the resource is notified from another resource.

#### Examples

##### Create a self-signed certificate with extensions valid for one year

```puppet

openssl_signcsr { '/tmp/cert.crt':
  csr        => '/tmp/cert.csr',
  signkey    => '/tmp/cert.key',
  extfile    => '/tmp/cert.cnf',
  extensions => 'v3_ext',
  days       => '365',
}
```

#### Properties

The following properties are available in the `openssl_selfsign` type.

##### `ensure`

Valid values: `present`, `absent`

The basic property that the resource should be in.

Default value: `present`

#### Parameters

The following parameters are available in the `openssl_selfsign` type.

* [`csr`](#-openssl_selfsign--csr)
* [`days`](#-openssl_selfsign--days)
* [`extensions`](#-openssl_selfsign--extensions)
* [`extfile`](#-openssl_selfsign--extfile)
* [`file`](#-openssl_selfsign--file)
* [`password`](#-openssl_selfsign--password)
* [`provider`](#-openssl_selfsign--provider)
* [`signkey`](#-openssl_selfsign--signkey)

##### <a name="-openssl_selfsign--csr"></a>`csr`

Required. The file containing the certificate signing request.

##### <a name="-openssl_selfsign--days"></a>`days`

Valid values: `%r{^[0-9]+$}`

The number of days the certificate should be valid.

Default value: `370`

##### <a name="-openssl_selfsign--extensions"></a>`extensions`

The section name of the extensions. The OpenSSL defaults will be used
if the parameter is `undef`.

##### <a name="-openssl_selfsign--extfile"></a>`extfile`

The file with the certificate extensions.

##### <a name="-openssl_selfsign--file"></a>`file`

The signed certificate file to manage.

##### <a name="-openssl_selfsign--password"></a>`password`

The password to decrypt the key.
Leave the parameter undefined if the key is not encrypted.

##### <a name="-openssl_selfsign--provider"></a>`provider`

The specific backend to use for this `openssl_selfsign` resource. You will seldom need to specify this --- Puppet will
usually discover the appropriate provider for your platform.

##### <a name="-openssl_selfsign--signkey"></a>`signkey`

Required. The file with the OpenSSL key to use for the self-signed certificate.

### <a name="openssl_signcsr"></a>`openssl_signcsr`

**This type is deprecated!**

The name and configuration file of a CA is required.

Certificate extensions can be added by using the `extensions` and
`extfile` parameters.

Optionally a key password can be provided if the used key is encrypted.

The certificate will be valid for the given number of days.

The type is refreshable. The `openssl_signcsr` type will regenerate the
certificate if the resource is notified from another resource.

#### Examples

##### Use a CA to sign a CSR

```puppet

openssl_signcsr { '/tmp/cert.crt':
  csr       => '/tmp/cert.csr',
  ca_name   => 'My-Root-CA',
  ca_config => '/etc/ssl/CA.cnf',
  days      => '365',
}
```

##### Regenerate a certificate if the CSR changes

```puppet

openssl_signcsr { '/tmp/cert.crt':
  csr       => '/tmp/cert.csr',
  ca_name   => 'My-Root-CA',
  ca_config => '/etc/ssl/CA.cnf',
  subscribe => File['/tmp/cert.csr'],
}
```

#### Properties

The following properties are available in the `openssl_signcsr` type.

##### `ensure`

Valid values: `present`, `absent`

The basic property that the resource should be in.

Default value: `present`

#### Parameters

The following parameters are available in the `openssl_signcsr` type.

* [`ca_config`](#-openssl_signcsr--ca_config)
* [`ca_name`](#-openssl_signcsr--ca_name)
* [`csr`](#-openssl_signcsr--csr)
* [`days`](#-openssl_signcsr--days)
* [`extensions`](#-openssl_signcsr--extensions)
* [`extfile`](#-openssl_signcsr--extfile)
* [`file`](#-openssl_signcsr--file)
* [`password`](#-openssl_signcsr--password)
* [`provider`](#-openssl_signcsr--provider)

##### <a name="-openssl_signcsr--ca_config"></a>`ca_config`

Required. The configuration file of the CA that is used to sign the CSR.

##### <a name="-openssl_signcsr--ca_name"></a>`ca_name`

Required. The name of the CA that is used to sign the CSR.

##### <a name="-openssl_signcsr--csr"></a>`csr`

Required. The file containing the certificate signing request.

##### <a name="-openssl_signcsr--days"></a>`days`

Valid values: `%r{^[0-9]+$}`

The number of days the certificate should be valid.

Default value: `370`

##### <a name="-openssl_signcsr--extensions"></a>`extensions`

The section name of the extensions. The OpenSSL defaults will be used
if the parameter is `undef`.

##### <a name="-openssl_signcsr--extfile"></a>`extfile`

The file with the certificate extensions.

##### <a name="-openssl_signcsr--file"></a>`file`

The signed certificate file to manage.

##### <a name="-openssl_signcsr--password"></a>`password`

The password to decrypt the CA key.
Leave the parameter undefined if the key is not encrypted.

##### <a name="-openssl_signcsr--provider"></a>`provider`

The specific backend to use for this `openssl_signcsr` resource. You will seldom need to specify this --- Puppet will
usually discover the appropriate provider for your platform.

## Data types

### <a name="Openssl--Extendedkeyusage"></a>`Openssl::Extendedkeyusage`

Valid parameter values for the OpenSSL extendend key usage

Alias of `Enum['serverAuth', 'clientAuth', 'codeSigning', 'emailProtection', 'timeStamping', 'OCSPSigning', 'ipsecIKE', 'msCodeInd', 'msCodeCom', 'msCTLSign', 'msEFS']`

### <a name="Openssl--Keyusage"></a>`Openssl::Keyusage`

Valid parameter values for the OpenSSL keyusage

Alias of `Enum['digitalSignature', 'nonRepudiation', 'keyEncipherment', 'dataEncipherment', 'keyAgreement', 'keyCertSign', 'cRLSign', 'encipherOnly', 'decipherOnly']`

